/*
This file is part of Slime Volley.

	Slime Volley is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	Slime Volley is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Slime Volley.  If not, see <http://www.gnu.org/licenses/>.

Copyright (c) MCMic, VinDuv.

$Id: objet.c 236 2008-11-01 09:45:35Z vinduv $
*/

#include <SDL.h>
#include <math.h>
#include "slime.h"
#include "objet.h"

slime_obj* cree_objet(Uint16 larg_surface, Uint16 decalage_larg, Uint16 haut_surface, SDL_Surface* obj_surface) {
/* Fonction permettant d'initialiser rapidement un objet slime. */

	slime_obj* nouv_objet = (slime_obj*) malloc(sizeof(slime_obj));

	nouv_objet->vy = 0;
	nouv_objet->vx = 0;
	nouv_objet->lim_bas = haut_surface - obj_surface->h;
	nouv_objet->lim_gauche = decalage_larg;
	nouv_objet->lim_droite = larg_surface - obj_surface->w + decalage_larg;
	nouv_objet->x = 0;
	nouv_objet->y = 0;
	nouv_objet->w = obj_surface->w;
	nouv_objet->h = obj_surface->h;
	nouv_objet->forme = obj_surface;

	return nouv_objet;
}

void libere_objet(struct slime_obj* objet) {
	SDL_FreeSurface(objet->forme);
	free(objet);
}

void place_objet(struct slime_obj* objet, Sint16 x, Sint16 y) {
	objet->x = (float)(x < 0 ? objet->lim_droite + 1 + x : x);
	objet->y = (float)(y < 0 ? objet->lim_bas + 1 + y : y);

	/* Cette fonction ne sert théoriquement qu'au départ ; On en profite donc pour remettre à 0 la vitesse. */
	objet->vy = 0;
	objet->vx = 0;
}

void deplace_joueur(Uint8 id, const Uint16 dt) {
/* Fonction basée sur certains cours de physique de terminale ;)    *
 * Elle permet de déplacer un objet en fonction du temps.           *
 * Attention, plus dt est grand plus l'approximation est mauvaise ! */

	if(dt > 250) return; /* Gros ralentissement (FPS < 4), on annule le déplacement */

	tab_joueurs[id].vy += conf.gravite * dt/1000;

	/* Mise à jour de la position... */
	tab_joueurs[id].x += (float)tab_joueurs[id].vx * dt/1000;
	tab_joueurs[id].y += (float)tab_joueurs[id].vy * dt/1000;

	/* On vérifie que le joueur ne sort pas du terrain. */
	if(tab_joueurs[id].x <= tab_joueurs[id].lim_gauche) {
		tab_joueurs[id].x = tab_joueurs[id].lim_gauche;
		tab_joueurs[id].vx = 0;
	} else if(tab_joueurs[id].x >= tab_joueurs[id].lim_droite) {
		tab_joueurs[id].x = tab_joueurs[id].lim_droite;
		tab_joueurs[id].vx = 0;
	}

	if(tab_joueurs[id].y >= tab_joueurs[id].lim_bas) {
    	tab_joueurs[id].y = tab_joueurs[id].lim_bas;
    	tab_joueurs[id].vx = 0;
    	tab_joueurs[id].vy = 0;
    }
}

void deplace_balle(Uint8 id, const Uint16 dt) {
	/* Equivalent strict de la fonction deplace_joueur() */
	if(dt > 250) return; /* Gros ralentissement (FPS < 4), on annule le déplacement */

	tab_balles[id].vy += conf.gravite * dt/1000;

	/* Mise à jour de la position... */
	tab_balles[id].x += (float)tab_balles[id].vx * dt/1000;
	tab_balles[id].y += (float)tab_balles[id].vy * dt/1000;
}

